package org.ow2.odis.joram;

import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.jms.BytesMessage;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageProducer;
import javax.jms.ObjectMessage;
import javax.jms.TextMessage;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.util.CodingDecoding;
import org.ow2.odis.util.IntegerModif;

/**
 * The Class AbstractConnexionJMSOut.
 * @author Beyler Olivier FranceTelecom R&D
 */
public abstract class AbstractConnectionJMSOut extends AbstractConnectionJMS {

    /**
     * Instantiates a new abstract connection JMS out.
     * @param attribut the attribute
     */
    public AbstractConnectionJMSOut(AbstractJMSConnectionAttribute attribute) {
        super(attribute);
    }
    /** logger of this class. */
    static final Logger LOGGER = Monolog.initialize().getLogger(AbstractConnectionJMSOut.class.getName());
    /** The sender. */
    private MessageProducer sender = null;

    /**
     * Send.
     * @param pMsg the message
     * @return true if succeed false in other case.
     */
    public boolean send(byte[] pMsg) {
        boolean result = true;
        if (getState() != AbstractConnectionJMS.READY) {
            if (LOGGER.isLoggable(BasicLevel.WARN)) {
                LOGGER.log(BasicLevel.WARN, "destination not ready " + attribute.getName());
            }
            result = false;
        }
        while (result && sender == null) {
            if (LOGGER.isLoggable(BasicLevel.WARN)) {
                LOGGER.log(BasicLevel.WARN, "destination not ready " + attribute.getName());
            }
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                if (LOGGER.isLoggable(BasicLevel.WARN)) {
                    LOGGER.log(BasicLevel.WARN, "InterruptedException", e);
                }
            }
        }
        if (result) {
            try {
                if (pMsg == null | pMsg.length == 0) {
                    return true;
                }
                if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                    StringBuffer msgDebug = new StringBuffer("try to send message to ");
                    msgDebug.append(attribute.getName());
                    LOGGER.log(BasicLevel.DEBUG, msgDebug);
                }
                List messages = new ArrayList();
                Message message = null;
                switch (attribute.getModeMessage()) {
                    case AbstractJMSConnectionAttribute.MODE_TEXT_MESSAGE:
                         {
                            List objects = attribute.getDecoder().decode(pMsg);
                            Iterator it = objects.iterator();
                            while (it.hasNext()) {
                                message = session.createTextMessage();
                                ((TextMessage) message).setText(it.next().toString());
                                messages.add(message);
                            }
                        }
                        break;
                    case AbstractJMSConnectionAttribute.MODE_PROPERTY_MESSAGE:
                         {
                            message = session.createMessage();
                            final IntegerModif index = new IntegerModif(0);
                            final StringBuffer text = new StringBuffer("");
                            while (index.intValue() != pMsg.length) {
                                try {
                                    text.append(CodingDecoding.getString(pMsg, index));
                                } catch (UnsupportedEncodingException e) {
                                    LOGGER.log(BasicLevel.ERROR, "UnsupportedEncodingException ", e);
                                }
                            }
                            message.setStringProperty("event", text.toString());
                            messages.add(message);
                        }
                        break;
                    case AbstractJMSConnectionAttribute.MODE_OBJECT_MESSAGE:
                         {
                            List objects = attribute.getDecoder().decode(pMsg);
                            Iterator it = objects.iterator();
                            while (it.hasNext()) {
                                message = session.createObjectMessage();
                                ((ObjectMessage) message).setObject((Serializable) it.next());
                                messages.add(message);
                            }
                        }
                        break;


                    case AbstractJMSConnectionAttribute.MODE_BYTE_MESSAGE:
                    // same case as default
                    default:
                        message = session.createBytesMessage();
                        ((BytesMessage) message).writeBytes(pMsg);
                        messages.add(message);
                        break;
                }
                if (getState() != AbstractConnectionJMS.READY) {
                    if (LOGGER.isLoggable(BasicLevel.WARN)) {
                        LOGGER.log(BasicLevel.WARN, "destination not ready " + attribute.getName());
                    }
                    return false;
                }
                Iterator it = messages.iterator();
                while (it.hasNext()) {
                    message = (Message) it.next();
                    sender.send(message);
                    if (LOGGER.isLoggable(BasicLevel.INFO)) {
                        LOGGER.log(BasicLevel.INFO, "message " + message.getJMSMessageID() + " sended to " + attribute.getName() + "length " + pMsg.length);
                    }
                }
            /*
             * TEST_NO_SESSION session.commit();
             */

            } catch (JMSException e) {
                // try to close connection and restore it
                LOGGER.log(BasicLevel.ERROR, "JMS Exception occured for to " + attribute.getName() + " ", e);
                LOGGER.log(BasicLevel.ERROR, "Retry to connect to " + attribute.getName() + " ", e);
                result = false;
            }
        }
        return true;
    }

    /**
     * Gets the sender.
     * @return the sender
     */
    public MessageProducer getSender() {
        return sender;
    }

    /**
     * Sets the sender.
     * @param sender the new sender
     */
    public void setSender(final MessageProducer sender) {
        this.sender = sender;
    }
}
