package org.ow2.odis.joram;

import java.io.IOException;
import java.io.OutputStream;
import java.util.List;

import org.jdom.Element;
import org.objectweb.util.monolog.api.BasicLevel;
import org.ow2.odis.decoder.IDecoder;
import org.ow2.odis.domain.Domain;
import org.ow2.odis.encoder.IEncoder;
import org.ow2.odis.exception.OdisAttributeException;
import org.ow2.odis.exception.OdisFactoryException;
import org.ow2.odis.model.AbstractAttribute;
import org.ow2.odis.model.AbstractConnectionAttribute;
import org.ow2.odis.model.Const;
import org.ow2.odis.util.CheckUtilities;

/**
 * The Class AbstractJMSCnxAttribut. Base class of JMS Connexion attribut.
 * @author Beyler Olivier FranceTelecom R&D
 */
public abstract class AbstractJMSConnectionAttribute extends AbstractConnectionAttribute implements AbstractJMSConnectionAttributeMBean {

    /**
     * Constructor.
     * @param parent AbstractAttribut
     */
    public AbstractJMSConnectionAttribute(AbstractAttribute parent) {
        super(parent);
    }
    /** The Constant PROPERTY_TYPE. */
    static final String PROPERTY_TYPE = "TYPE";
    /** The Constant PROPERTY_PROXY_HOSTNAME. */
    static final String PROPERTY_PROXY_HOSTNAME = "PROXY_HOSTNAME";
    /** The Constant PROPERTY_PROXY_PORT. */
    static final String PROPERTY_PROXY_PORT = "PROXY_PORT";
    /** The Constant PROPERTY_SERVER_ID. */
    static final String PROPERTY_SERVER_ID = "SERVER_ID";
    /** The Constant PROPERTY_HA_CLUSTER. */
    static final String PROPERTY_HA_CLUSTER = "HA_CLUSTER";
    /** The Constant PROPERTY_USER_NAME. */
    static final String PROPERTY_USER_NAME = "USER_NAME";
    /** The Constant PROPERTY_USER_PASSWORD. */
    static final String PROPERTY_USER_PASSWORD = "USER_PASSWORD";
    /** The Constant TYPE_UNDEFINED. */
    public static final int TYPE_UNDEFINED = 0;
    /** The Constant TYPE_QUEUE. */
    public static final int TYPE_QUEUE = 1;
    /** The Constant TYPE_TOPIC. */
    public static final int TYPE_TOPIC = 2;
    /** String constant. */
    public static final String PROPERTY_MODE_MESSAGE = "MODE";
    /** The Constant PROPERTY_JMS_NAME. */
    public static final String PROPERTY_JMS_NAME = "JMS_NAME";
    /** The Constant PROPERTY_MODE_BYTE_MESSAGE. */
    public static final String PROPERTY_MODE_BYTE_MESSAGE = "BYTE_MESSAGE";
    /** The Constant PROPERTY_MODE_TEXT_MESSAGE. */
    public static final String PROPERTY_MODE_TEXT_MESSAGE = "TEXT_MESSAGE";
    /** The Constant PROPERTY_MODE_PROPERTY_MESSAGE. */
    public static final String PROPERTY_MODE_PROPERTY_MESSAGE = "PROPERTY_MESSAGE";
    /** The Constant PROPERTY_MODE_OBJECT_MESSAGE. */
    public static final String PROPERTY_MODE_OBJECT_MESSAGE = "OBJECT_MESSAGE";
    public static final String PROPERTY_DECODER = "DECODER";
    public static final String PROPERTY_ENCODER = "ENCODER";
    /** The Constant MODE_BYTE_MESSAGE. */
    public static final int MODE_BYTE_MESSAGE = 0;
    /** The Constant MODE_TEXT_MESSAGE. */
    public static final int MODE_TEXT_MESSAGE = 1;
    /** The Constant MODE_PROPERTY_MESSAGE. */
    public static final int MODE_PROPERTY_MESSAGE = 2;
    /** The Constant MODE_PROPERTY_MESSAGE. */
    public static final int MODE_OBJECT_MESSAGE = 3;
    /** The mode message. */
    private int modeMessage = MODE_BYTE_MESSAGE;
    private String strModeMessage = "BYTE_MESSAGE";

    public String getStrModeMessage() {
        return strModeMessage;
    }
    /** The server id. */
    private int serverId;
    /** The string representing the server id. */
    private String strServerId = "0";
    /** The jms name. */
    private String jmsName = "";
    /** The proxy server port. */
    private int proxyServerPort = 16010;
    /** The string representing proxy server port. */
    private String strProxyServerPort = "16010";
    /** The proxy server host name. */
    private String proxyServerHostName = "localhost";
    /** The user name. */
    private String userName = "root";
    /** The user password. */
    private String userPassword = "root";
    /** The HA cluster description. */
    private String HAClusterDescription = "";
    /** The nbr reconnexion. */
    private int nbrReconnexion = 0;
    private String strDecode = "";
    private String strEncode = "";
    /**
     * Inc nbr reconnexion.
     * @param value the value
     */
    public void incNbrReconnexion(int value) {
        nbrReconnexion += value;
    }
    /** do not close JMS connection if nbSession > 1. */
    private int nbSession = 0;
    private IDecoder decode;
    private IEncoder encode;

    /**
     * Setter JMS.
     * @param cnxJMSDeclaration the cnx JMS declaration
     */
    public void subSetter(Element cnxJMSDeclaration) {
        List lsUnusedProperties = cnxJMSDeclaration.getChildren();
        Element element = cnxJMSDeclaration.getChild(PROPERTY_PROXY_HOSTNAME);
        if (element != null) {
            proxyServerHostName = element.getAttributeValue(Const.VALUE, proxyServerHostName);
            lsUnusedProperties.remove(element);
        }
        element = cnxJMSDeclaration.getChild(PROPERTY_PROXY_PORT);
        if (element != null) {
            strProxyServerPort = element.getAttributeValue(Const.VALUE, strProxyServerPort);
            lsUnusedProperties.remove(element);
        }
        element = cnxJMSDeclaration.getChild(PROPERTY_SERVER_ID);
        if (element != null) {
            strServerId = element.getAttributeValue(Const.VALUE, strServerId);
            lsUnusedProperties.remove(element);
        }
        element = cnxJMSDeclaration.getChild(PROPERTY_USER_NAME);
        if (element != null) {
            userName = element.getAttributeValue(Const.VALUE, userName);
            lsUnusedProperties.remove(element);
        }
        element = cnxJMSDeclaration.getChild(PROPERTY_USER_PASSWORD);
        if (element != null) {
            userPassword = element.getAttributeValue(Const.VALUE, userPassword);
            lsUnusedProperties.remove(element);
        }
        element = cnxJMSDeclaration.getChild(PROPERTY_MODE_MESSAGE);
        if (element != null) {
            strModeMessage = element.getAttributeValue(Const.VALUE, strModeMessage);
            lsUnusedProperties.remove(element);
        }
        element = cnxJMSDeclaration.getChild(PROPERTY_JMS_NAME);
        if (element != null) {
            jmsName = element.getAttributeValue(Const.VALUE, jmsName);
            lsUnusedProperties.remove(element);
        }
        element = cnxJMSDeclaration.getChild(PROPERTY_HA_CLUSTER);
        if (element != null) {
            HAClusterDescription = element.getAttributeValue(Const.VALUE, HAClusterDescription);
            lsUnusedProperties.remove(element);
        }
        element = cnxJMSDeclaration.getChild(PROPERTY_DECODER);
        if (element != null) {
            strDecode = element.getAttributeValue(Const.VALUE, strDecode);
            lsUnusedProperties.remove(element);
        }
        
        element = cnxJMSDeclaration.getChild(PROPERTY_ENCODER);
        if (element != null) {
            strEncode = element.getAttributeValue(Const.VALUE, strEncode);
            lsUnusedProperties.remove(element);
        }

        
        CheckUtilities.checkUnusedProperties(lsUnusedProperties, LOGGER);
    }

    /**
     * Check properties.
     * @throws OdisAttributException the odis attribut exception
     * @see com.francetelecom.odis.model.AbstractAttribut#checkProperties()
     */
    public void checkProperties() throws OdisAttributeException {
        StringBuffer buff = null;
        try {
            setProxyServerPort(Integer.parseInt(strProxyServerPort));
        } catch (NumberFormatException e) {
            buff = concatException(buff, PROPERTY_PROXY_PORT, strProxyServerPort, e);
        }
        try {
            if (!"".equals(strDecode)) {
                setDecoder(strDecode);
            }
        } catch (Exception e) {
            buff = concatException(buff, PROPERTY_DECODER, strDecode, e);
        }
        try {
            if (!"".equals(strEncode)) {
                setEncoder(strEncode);
            }
        } catch (Exception e) {
            buff = concatException(buff, PROPERTY_DECODER, strDecode, e);
        }
        try {
            setModeMessage(strModeMessage);
            if (modeMessage == MODE_OBJECT_MESSAGE && decode == null) {
                throw new OdisAttributeException("MODE OBJECT without DECODER ");
            }
            if (modeMessage == MODE_TEXT_MESSAGE && decode == null) {
                throw new OdisAttributeException("MODE TEXT without DECODER ");
            }
            if (modeMessage == MODE_OBJECT_MESSAGE && encode == null) {
                throw new OdisAttributeException("MODE OBJECT without ENCODER ");
            }
            if (modeMessage == MODE_TEXT_MESSAGE && encode == null) {
                throw new OdisAttributeException("MODE TEXT without ENCODER ");
            }
            
        } catch (OdisAttributeException e) {
            buff = concatException(buff, PROPERTY_MODE_MESSAGE, strModeMessage, e);
        }
        try {
            setServerId(Integer.parseInt(strServerId));
        } catch (NumberFormatException e) {
            buff = concatException(buff, PROPERTY_SERVER_ID, strServerId, e);
        }


        if (buff != null) {
            LOGGER.log(BasicLevel.ERROR, buff.toString());
            throw new OdisAttributeException(buff.toString());
        }
    }

    public final IDecoder getDecoder() {
        return decode;
    }
    
    public final IEncoder getEncoder() {
        return encode;
    }

    final void setDecoder(String decoder) throws OdisFactoryException {
        Domain d = Domain.getInstance();
        decode = (IDecoder) d.getFactoryInstance().getInstanceOf(decoder, "1");
    }

    final void setEncoder(String encoder) throws OdisFactoryException {
        Domain d = Domain.getInstance();
        encode = (IEncoder) d.getFactoryInstance().getInstanceOf(encoder, "1");
    }
    
    /**
     * Sets the mode message.
     * @param value the new mode message
     * @throws OdisAttributeException the odis attribute exception
     */
    public void setModeMessage(String value) throws OdisAttributeException {
        if (PROPERTY_MODE_BYTE_MESSAGE.equals(value)) {
            modeMessage = MODE_BYTE_MESSAGE;
        } else if (PROPERTY_MODE_TEXT_MESSAGE.equals(value)) {
            modeMessage = MODE_TEXT_MESSAGE;
        } else if (PROPERTY_MODE_PROPERTY_MESSAGE.equals(value)) {
            modeMessage = MODE_PROPERTY_MESSAGE;
        } else if (PROPERTY_MODE_OBJECT_MESSAGE.equals(value)) {
            modeMessage = MODE_OBJECT_MESSAGE;
        } else if (PROPERTY_MODE_BYTE_MESSAGE.equals(value)) {
            modeMessage = MODE_BYTE_MESSAGE;
        } else {
            throw new OdisAttributeException("unknown mode");
        }
    }

    /**
     * Equals.
     * As tho connextion with the same name should have the same description,
     * the equals function will log as a warning the difference with the copy
     * if only the name are equals.
     * @param obj to be compare
     * @return boolean
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        boolean result = false;
        AbstractJMSConnectionAttribute copy = null;
        try {
            copy = (AbstractJMSConnectionAttribute) obj;
        } catch (ClassCastException e) {
            return false;
        }
        result = super.equals(obj);
        if (result) {
            result &= (copy.HAClusterDescription == null && HAClusterDescription == null) ||
                    copy.HAClusterDescription.equals(this.HAClusterDescription);
            result &= (copy.jmsName == null && jmsName == null) ||
                    copy.jmsName.equals(this.jmsName);
            result &= (copy.proxyServerHostName == null && proxyServerHostName == null) ||
                    copy.proxyServerHostName.equals(this.proxyServerHostName);
            result &= (copy.strProxyServerPort == null && strProxyServerPort == null) ||
                    copy.strProxyServerPort.equals(this.strProxyServerPort);
            result &= (copy.userName == null && userName == null) ||
                    copy.userName.equals(this.userName);
            result &= (copy.userPassword == null && userPassword == null) ||
                    copy.userPassword.equals(this.userPassword);
            result &= (copy.strServerId == null && strServerId == null) ||
                    copy.strServerId.equals(this.strServerId);
            result &= (copy.modeMessage == this.modeMessage);

            if (!result) {
                traceOnConfictualConnection(copy);
            }
        }
        return result;
    }

    /**
     * Gets the mode message.
     * @return the mode message
     * @see com.AbstractJMSConnectionAttributeMBean.odis.model.AbstractJMSCnxAttributMBean#getModeMessage()
     */
    public int getModeMessage() {
        return modeMessage;
    }

    /**
     * Gets the proxy server host name.
     * @return Retourne proxyServerHostName.
     */
    public String getProxyServerHostName() {
        return proxyServerHostName;
    }

    /**
     * Sets the proxy server host name.
     * @param pProxyServerHostName affecte proxyServerHostName.
     */
    public void setProxyServerHostName(String pProxyServerHostName) {
        proxyServerHostName = pProxyServerHostName;
    }

    /**
     * Gets the proxy server port.
     * @return Retourne proxyServerPort.
     */
    public int getProxyServerPort() {
        return proxyServerPort;
    }

    /**
     * Sets the proxy server port.
     * @param pProxyServerPort affecte proxyServerPort.
     */
    public void setProxyServerPort(int pProxyServerPort) {
        proxyServerPort = pProxyServerPort;
    }

    /**
     * Gets the server id.
     * @return Retourne serverId.
     */
    public int getServerId() {
        return serverId;
    }

    /**
     * Sets the server id.
     * @param pServerId affecte serverId.
     */
    public void setServerId(int pServerId) {
        serverId = pServerId;
    }

    /**
     * Gets the user name.
     * @return the user name
     * @see com.AbstractJMSConnectionAttributeMBean.odis.model.AbstractJMSCnxAttributMBean#getUserName()
     */
    public String getUserName() {
        return userName;
    }

    /**
     * Sets the user name. *
     * @param userName the new user name
     */
    public void setUserName(String userName) {
        this.userName = userName;
    }

    /**
     * Gets the user password.
     * @return the user password
     * @see com.AbstractJMSConnectionAttributeMBean.odis.model.AbstractJMSCnxAttributMBean#getUserPassword()
     */
    public String getUserPassword() {
        return userPassword;
    }

    /**
     * Sets the user password. *
     * @param userPassword the new user password
     */
    public void setUserPassword(String userPassword) {
        this.userPassword = userPassword;
    }

    /**
     * Sets the name.
     * @param pName the name
     */
    public void setName(String pName) {
        super.setName(pName);
    }

    /**
     * Sets the JMS name. *
     * @param pName the new JMS name
     */
    public void setJMSName(String pName) {
        jmsName = pName;
    }

    /**
     * Gets the JMS name.
     * @return the JMS name
     * @see com.AbstractJMSConnectionAttributeMBean.odis.model.AbstractJMSCnxAttributMBean#getJMSName()
     */
    public String getJMSName() {
        return jmsName;
    }

    /**
     * Gets the nbr reconnexion.
     * @return the nbr reconnexion
     * @see com.AbstractJMSConnectionAttributeMBean.odis.model.AbstractJMSCnxAttributMBean#getNbrReconnexion()
     */
    public int getNbrReconnexion() {
        return nbrReconnexion;
    }

    /**
     * Write XML.
     * @param out the out
     * @throws IOException Signals that an I/O exception has occurred.
     * @see com.francetelecom.odis.model.AbstractAttribut#writeXML(java.io.OutputStream)
     */
    public void writeXML(OutputStream out) throws IOException {
        writeXML(out, false);
    }

    /**
     * Write XML.
     * @param out the out
     * @param full the full
     * @throws IOException Signals that an I/O exception has occurred.
     * @see com.francetelecom.odis.model.AbstractAttribut#writeXML(java.io.OutputStream, boolean)
     */
    public void writeXML(OutputStream out, boolean full) throws IOException {
        out.write((Const.TAB + "<"  + getTypeConnection() + " name=\"" + getName() + "\">" + Const.EOL).getBytes());
        if (!"".equals(jmsName)) {
            writeXMLCNXUtilities(out, 2, PROPERTY_JMS_NAME, getJMSName(), getName(), full);
        }
        writeXMLCNXUtilities(out, 2, PROPERTY_SERVER_ID, "" + getServerId(), null, full);
        if (!"".equals(getHAClusterDescription())) {
            writeXMLCNXUtilities(out, 2, PROPERTY_HA_CLUSTER, getHAClusterDescription(), null, full);
        }
        writeXMLCNXUtilities(out, 2, PROPERTY_PROXY_HOSTNAME, getProxyServerHostName(), null, full);
        writeXMLCNXUtilities(out, 2, PROPERTY_PROXY_PORT, "" + getProxyServerPort(), null, full);
        writeXMLCNXUtilities(out, 2, PROPERTY_USER_NAME, getUserName(), null, full);
        writeXMLCNXUtilities(out, 2, PROPERTY_USER_PASSWORD, getUserPassword(), null, full);
        out.write((Const.TAB + "</" + getTypeConnection() + ">" + Const.EOL).getBytes());
    }

    /**
     * Gets the HA cluster description.
     * @return the HA cluster description
     */
    public String getHAClusterDescription() {
        return HAClusterDescription;
    }

    /**
     * Sets the HA cluster description. *
     * @param clusterDescription the new HA cluster description
     */
    public void setHAClusterDescription(String clusterDescription) {
        HAClusterDescription = clusterDescription;
    }

    /**
     * get number of session for this connection.
     * @return number of session
     */
    public int getNbSession() {
        return nbSession;
    }

    /**
     * nbSession++.
     */
    public synchronized void addSession() {
        nbSession++;
        if (LOGGER.isLoggable(BasicLevel.INFO)) {
            LOGGER.log(BasicLevel.INFO, "addSession nbSession = " + nbSession + " on " + this);
        }
    }

    /**
     * nbSession--.
     */
    public synchronized void removeSession() {
        nbSession--;
        if (LOGGER.isLoggable(BasicLevel.INFO)) {
            LOGGER.log(BasicLevel.INFO, "removeSession nbSession = " + nbSession + " on " + this);
        }
    }

    /**
     * Sets the nbr reconnexion. *
     * @param nbrReconnexion the new nbr reconnexion
     */
    public void setNbrReconnexion(int nbrReconnexion) {
        this.nbrReconnexion = nbrReconnexion;
    }
}
