package org.ow2.odis.joram;

import java.net.ConnectException;

import javax.jms.JMSException;
import javax.jms.Session;

import org.objectweb.joram.client.jms.Queue;
import org.objectweb.joram.client.jms.admin.AdminException;
import org.objectweb.joram.client.jms.admin.AdminModule;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.connection.ConnectionManager;
import org.ow2.odis.connection.IConnectionIn;
import org.ow2.odis.port.PortIn;

/**
 * The Class InQueue.
 * @author Olivier Beyler
 */
public class InQueue extends AbstractConnectionJMSIn implements IConnectionIn {
    /** The Constant LOGGER. */
    static final Logger LOGGER = Monolog.initialize().getLogger(InQueue.class.getName());

    /**
     * Instantiates a new in queue.
     * @param attribute the attribute
     */
    public InQueue(AbstractJMSConnectionAttribute attribute) {
        super(attribute);
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            LOGGER.log(BasicLevel.DEBUG, "InQueue Constructor");
        }
    }

    /**
     * retrieve or create the destination.
     * @throws ConnectException the connect exception
     * @throws AdminException the admin exception
     * @throws JMSException the JMS exception
     */
    void specificCreateDestination() throws ConnectException, AdminException, JMSException {
        if (destination == null) {
            if ("".equals(attribute.getJMSName())) {
                if (LOGGER.isLoggable(BasicLevel.INFO)) {
                    LOGGER.log(BasicLevel.INFO, "Creating IN queue by Name : " + attribute.getName());
                }
                destination = Queue.create(attribute.getServerId(), attribute.getName());
            } else {
                if (LOGGER.isLoggable(BasicLevel.INFO)) {
                    LOGGER.log(BasicLevel.INFO, "Creating IN queue by JMSName : " + attribute.getJMSName());
                }
                destination = Queue.create(attribute.getServerId(), attribute.getJMSName());
            }
            destination.setFreeReading();
            if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                LOGGER.log(BasicLevel.DEBUG, "[" + destination + "] free reading right set");
            }
            if (session == null) {
                if (LOGGER.isLoggable(BasicLevel.INFO)) {
                    LOGGER.log(BasicLevel.INFO, "Creating session");
                }
                session = cnx.createSession(false, Session.CLIENT_ACKNOWLEDGE);
                if (session != null) {
                    attribute.addSession();
                } else {
                    LOGGER.log(BasicLevel.ERROR, "Couldn't create session");
                }
            }
            setReceiver(session.createConsumer(destination));
            getReceiver().setMessageListener(new MsgListener(this));
            if (LOGGER.isLoggable(BasicLevel.INFO)) {
                LOGGER.log(BasicLevel.INFO, "Disconnecting from AdminModule");
            }
            AdminModule.disconnect();
        }
    }

    /**
     * set the port in.
     * @param portIn the port in
     * @return IConnexionIn
     * @see com.francetelecom.odis.component.connexion.jms.AbstractConnectionJMSIn#setPortIn(com.francetelecom.odis.component.port.PortIn)
     */
    public IConnectionIn setPortIn(final PortIn portIn) {
        this.portIn = portIn;
        ConnectionManager.getInstance().addConnectionIn(this, portIn);
        return this;
    }
}
