package org.ow2.odis.joram;

import java.net.ConnectException;

import javax.jms.JMSException;
import javax.jms.MessageConsumer;

import org.objectweb.joram.client.jms.Session;
import org.objectweb.joram.client.jms.Topic;
import org.objectweb.joram.client.jms.admin.AdminException;
import org.objectweb.joram.client.jms.admin.AdminModule;
import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.odis.connection.ConnectionManager;
import org.ow2.odis.connection.IConnectionIn;
import org.ow2.odis.port.PortIn;


/**
 * Class aim of InTopic is to be able to receive message from a topic.
 * @author Olivier Beyler
 */
public class InTopic extends AbstractConnectionJMSIn implements IConnectionIn {
    /** Logger of this class. */
    static final Logger LOGGER = Monolog.initialize().getLogger(InTopic.class.getName());

    /**
     * Instantiates a new in topic.
     * @param attribute the attribute
     */
    public InTopic(AbstractJMSConnectionAttribute attribute) {
        super(attribute);
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            LOGGER.log(BasicLevel.DEBUG, "create InTopic " + this);
        }
    }

    /*
     * Constructor by copy. @param copy InTopic clone. @param attribut
     */
    /**
     * Instantiates a new in topic.
     * @param copy the copy
     */
    public InTopic(InTopic copy) {
        super(copy.attribute);
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            LOGGER.log(BasicLevel.DEBUG, "create InTopic by copy " + this);
        }
    }

    /**
     * retrieve or create the destination.
     * @throws ConnectException the connect exception
     * @throws AdminException the admin exception
     * @throws JMSException the JMS exception
     */
    void specificCreateDestination() throws ConnectException, AdminException, JMSException {
        if (destination == null) {
            if ("".equals(attribute.getJMSName())) {
                destination = Topic.create(attribute.getServerId(), attribute.getName());
            } else {
                destination = Topic.create(attribute.getServerId(), attribute.getJMSName());
            }
            destination.setFreeReading();
            if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                LOGGER.log(BasicLevel.DEBUG, "[" + destination + "] free reading right set");
            }
            if (session == null) {
                session = cnx.createSession(false, Session.CLIENT_ACKNOWLEDGE);
                if (session != null) {
                    attribute.addSession();
                }
            }
            if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                LOGGER.log(BasicLevel.DEBUG, "session [" + session.toString() + "] is created");
            }
            MessageConsumer consummer = session.createConsumer(destination);
            if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                LOGGER.log(BasicLevel.DEBUG, "consummer [" + consummer.toString() + "] is created");
            }
            setReceiver(consummer);
            getReceiver().setMessageListener(new MsgListener(this));
        }
        if (LOGGER.isLoggable(BasicLevel.INFO)) {
            LOGGER.log(BasicLevel.INFO, "Disconnecting from AdminModule");
        }
        AdminModule.disconnect();
    }

    /**
     * specific setter. if the topic has already a link a copy is created to set this new link. <br>
     * The reason of this behaviours id : only once link by Intopic is allowed. <br>
     * Each link will have his own session.
     * @param portIn that need to be notify when a new Message happend.
     * @return InTopic instance.
     */
    public IConnectionIn setPortIn(PortIn portIn) {
        if (this.portIn != null) {
            InTopic topic = new InTopic(this);
            return topic.setPortIn(portIn);
        }
        this.portIn = portIn;
        ConnectionManager.getInstance().addConnectionIn(this, portIn);
        return this;
    }
}
