/*--------------------------------------------------------
 * FTR&D/DMI/MAPS/AMS
 *--------------------------------------------------------
 * Project     : M2MPlanet
 * SubProject  : ODIS2
 *
 * Copyright France Telecom  2005,  All Rights Reserved.
 *
 * This software is the confidential and proprietary
 * information of France Telecom.
 * You shall not disclose such Confidential Information
 * and shall use it only in accordance with the terms
 * of the license agreement you entered into with
 * France Telecom.
 *
 *--------------------------------------------------------
 * FileName    : JMSExceptionListener.java
 *
 * Created     : 27 sept. 2005
 * Author      : Olivier Beyler
 *
 * Description :
 * [Description sommaire]
 *
 *--------------------------------------------------------
 * $Log$
 *
 */
package org.ow2.odis.joram;

import javax.jms.Connection;
import javax.jms.ExceptionListener;
import javax.jms.JMSException;

import org.objectweb.util.monolog.Monolog;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * The listener interface for receiving JMSException events. The class that is interested in processing a JMSException event implements
 * this interface, and the object created with that class is registered with a component using the component's
 * <code>addJMSExceptionListener<code> method. When
 * the JMSException event occurs, that object's appropriate
 * method is invoked.
 *
 * @see JMSExceptionEvent
 */
public class JMSExceptionListener implements ExceptionListener {
    /** The Constant LOGGER. */
    static final Logger LOGGER = Monolog.initialize().getLogger(JMSExceptionListener.class.getName());

    /** The connection. */
    Connection connection = null;

    /** The old. */
    ExceptionListener old = null;

    /** The nbr. */
    static int nbr = 0;

    /** The compteur. */
    static int compteur = 0;

    /**
     * Instantiates a new JMS exception listener.
     * @param connection the connection
     * @param old the old
     */
    JMSExceptionListener(Connection connection, ExceptionListener old) {
        super();
        this.connection = connection;
        this.old = old;
        nbr++;
    }

    /**
     * @param arg0
     * @see javax.jms.ExceptionListener#onException(javax.jms.JMSException)
     */
    public void onException(JMSException arg0) {
        if (old != null) {
            old.onException(arg0);
        }
        LOGGER.log(BasicLevel.WARN, "JMSException",arg0);
        compteur = nbr;
        ConnectionJMSManager manager = ConnectionJMSManager.getInstance();
        manager.suspendCnx(connection);
        boolean connexionRestore = false;
        while (connexionRestore != true) {
            try {
                if (LOGGER.isLoggable(BasicLevel.WARN)) {
                    LOGGER.log(BasicLevel.WARN, " try to restore jms connexion failed retry in 10 s");
                }
                Thread.sleep(10000);
                if (LOGGER.isLoggable(BasicLevel.WARN)) {
                    LOGGER.log(BasicLevel.WARN, " retry restore jms connexion");
                }
                connexionRestore = manager.restoreConnexion(connection) == 0;
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
        if (LOGGER.isLoggable(BasicLevel.WARN)) {
            LOGGER.log(BasicLevel.WARN, " try to restore destination succced");
        }
    }
}
