package org.ow2.odis.joram;

import java.net.ConnectException;

import javax.jms.JMSException;
import javax.jms.Session;

import org.objectweb.joram.client.jms.Queue;
import org.objectweb.joram.client.jms.admin.AdminException;
import org.objectweb.joram.client.jms.admin.AdminModule;
import org.objectweb.util.monolog.api.BasicLevel;
import org.ow2.odis.connection.IConnectionOut;

/**
 * The Class OutQueue.
 * @author Olivier Beyler
 */
public class OutQueue extends AbstractConnectionJMSOut implements IConnectionOut {
    /**
     * Instantiates a new out queue.
     * @param attribute the attribute
     */
    public OutQueue(AbstractJMSConnectionAttribute attribute) {
        super(attribute);
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            LOGGER.log(BasicLevel.DEBUG, "OutQueue is created");
        }
    }

    /**
     * Specific create destination.
     * @throws AdminException the admin exception
     * @throws ConnectException the connect exception
     * @throws JMSException the JMS exception
     * @see com.francetelecom.odis.component.connexion.jms.AbstractConnectionJMS#specificCreateDestination()
     */
    void specificCreateDestination() throws ConnectException, AdminException, JMSException {
        if ("".equals(attribute.getJMSName())) {
            destination = Queue.create(attribute.getServerId(), attribute.getName());
        } else {
            destination = Queue.create(attribute.getServerId(), attribute.getJMSName());
        }
        destination.setFreeWriting();
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            LOGGER.log(BasicLevel.DEBUG, "[" + destination + "] free writing right set");
        }
        if (session == null) {
            session = cnx.createSession(false, Session.CLIENT_ACKNOWLEDGE);
            if (session != null) {
                attribute.addSession();
            }
        }
        setSender(session.createProducer(destination));
        if (LOGGER.isLoggable(BasicLevel.INFO)) {
            LOGGER.log(BasicLevel.INFO, "Disconnecting from AdminModule");
        }
        AdminModule.disconnect();
    }

    /**
     * @see com.francetelecom.odis.component.connexion.jms.AbstractConnectionJMS#stopComponent()
     */
    public void stopComponent() {
        super.stopComponent();
    }
}
