package org.ow2.odis.joram;

import java.net.ConnectException;

import javax.jms.JMSException;
import javax.jms.Session;

import org.objectweb.joram.client.jms.Topic;
import org.objectweb.joram.client.jms.admin.AdminException;
import org.objectweb.joram.client.jms.admin.AdminModule;
import org.objectweb.util.monolog.api.BasicLevel;
import org.ow2.odis.connection.IConnectionOut;


/**
 * Class aim of oUTTopic is to be able to send message to a topic.
 * @author Olivier Beyler
 */
public class OutTopic extends AbstractConnectionJMSOut implements IConnectionOut {
    /**
     * Instantiates a new out topic.
     * @param attribut the attribute
     */
    public OutTopic(AbstractJMSConnectionAttribute attribut) {
        super(attribut);
        if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
            LOGGER.log(BasicLevel.DEBUG, "OutTopic is created");
        }
    }

    /**
     * Specific create destination.
     * @throws ConnectException the connect exception
     * @throws AdminException the admin exception
     * @throws JMSException the JMS exception
     * @see com.francetelecom.odis.component.connexion.jms.AbstractConnectionJMS#specificCreateDestination()
     */
    void specificCreateDestination() throws ConnectException, AdminException, JMSException {
        if (destination == null) {
            if ("".equals(attribute.getJMSName())) {
                destination = Topic.create(attribute.getServerId(), attribute.getName());
            } else {
                destination = Topic.create(attribute.getServerId(), attribute.getJMSName());
            }
            destination.setFreeWriting();
            if (LOGGER.isLoggable(BasicLevel.DEBUG)) {
                LOGGER.log(BasicLevel.DEBUG, "[" + destination + "] free writing right set");
            }
            if (session == null) {
                session = cnx.createSession(false, Session.CLIENT_ACKNOWLEDGE);
                if (session != null) {
                    attribute.addSession();
                }
            }
            setSender(session.createProducer(destination));
            if (LOGGER.isLoggable(BasicLevel.INFO)) {
                LOGGER.log(BasicLevel.INFO, "Disconnecting from AdminModule");
            }
            AdminModule.disconnect();
        }
    }

    /**
     * Stop component.
     * @see com.francetelecom.odis.component.connexion.jms.AbstractConnectionJMS#stopComponent()
     */
    public void stopComponent() {
        super.stopComponent();
    }
}
